import math
import random

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import levelObjects
import playerMgr
import statePauseMenu

END_TUTORIAL_LEVEL=4

END_COURSE_LEVELS=[28,48,68]

class StateGame(fsmMgr.State):
    def __init__(self, parentState, data):
        #parentState is None, right?
        self.angle=0
        self.power=0
        self.accuracy=0

        self.actualAngle=0
        self.actualPower=0

        self.setLevel(1)

        self.gameInPlay=True
        self.displayList=None

    def pickRandomLevel(self):
        return self.setLevel(random.randrange(10,28))
        
    def nextLevel(self):
        return self.setLevel(self.level+1)

    def setLevel(self, levelNum):
        #pygame.display.set_caption("level %d"%levelNum)
        
        self.level=levelNum
        if self.level==END_TUTORIAL_LEVEL:
            return "end_tutorial"
        if self.level in END_COURSE_LEVELS:
            return "end_course"

        self.physicsObjects=levelObjects.getObjects(self.level)
        self.displayList=None

        self.levelTime=0.0
        playerMgr.newLevel(self.physicsObjects[0],self.level)
        return None
        

    def update(self, ms):
        self.levelTime += ms/1000.0

        for p in self.physicsObjects:
            parentName=p.get("orbits",None)
            if not (parentName is None):
                for parent in self.physicsObjects:
                    if parent["name"]==parentName:
                        orbitRate=p.get("orbitRate")
                        timeIntoOrbit=math.fmod(self.levelTime,abs(orbitRate))
                        fracAroundOrbit=timeIntoOrbit/orbitRate+p["orbitOffset"]
                        angle=math.pi*2.0*fracAroundOrbit
                        orbRadius=p["orbitRadius"]
                        pcx,pcy=parent["center"]
                        p["center"]=(pcx+math.cos(angle)*orbRadius,
                                     pcy+math.sin(angle)*orbRadius)

        

    def renderObject(self, p, physicsObjects):
        cx,cy=p["center"]
        r=p["radius"]
        color=p["color"]

        glColor3f(*color)

        if p.get("dust", False):
            glBegin(GL_POINTS)
        else:
            glBegin(GL_LINE_LOOP)

        NUM_STEPS=30
        inc_angle=math.pi*2.0/float(NUM_STEPS)

        for i in range(NUM_STEPS):
            a=i*inc_angle
            dx=math.cos(a)*r
            dy=math.sin(a)*r

            nx=cx+dx
            ny=cy+dy

            if p.get("dust", False):
                inside=False
                for otherObj in physicsObjects:
                    if otherObj["name"]==p["name"]:
                        continue
                    ox,oy=otherObj["center"]
                    rad=otherObj["radius"]
                    dx=ox-nx
                    dy=oy-ny

                    if dx*dx+dy*dy<rad*rad:
                        inside=True
                        break
                if inside:
                    continue

            glVertex2f(nx, ny)

        glEnd()

        if p.get("target",False):
            off=r*0.65
            glBegin(GL_LINES)
            glVertex2f(cx-off,cy-off)
            glVertex2f(cx+off,cy+off)
            glVertex2f(cx-off,cy+off)
            glVertex2f(cx+off,cy-off)
            glEnd()

        if p.get("gaseous",False):
            wiggleData=[(.1, .80),
                        (.2, .72),
                        (.3, .70),
                        (.4, .67),
                        (.5, .69),
                        (.6, .74),
                        (.7, .80),
                        (.8, .80),
                        (.9, .74),
                        (.95, .71)]

            for deltaY in [0,-.4]:
                glBegin(GL_LINE_STRIP)
                for wx,wy in wiggleData:
                    s=2*r
                    glVertex(cx-r+s*wx,
                             cy-r+s*(wy+deltaY))

                glEnd()

        
        
    def draw(self):
        if not self.displayList:
            self.displayList=glGenLists(1)
            glNewList(self.displayList, GL_COMPILE)
            
            for p in self.physicsObjects:
                if not p.get("orbits",None):
                    self.renderObject(p, self.physicsObjects)
            glEndList()

        glCallList(self.displayList)
        for p in self.physicsObjects:
            if p.get("orbits",None):
                self.renderObject(p, self.physicsObjects)
        
                    

    def activateApp(self, bNowActive):
        if not bNowActive and self.gameInPlay:
            self.gameInPlay=False
            fsmMgr.pushState(statePauseMenu.StatePauseMenu(None))
            return True

    def handleKey(self, key, unicode):
        if key==K_ESCAPE and self.gameInPlay:
            self.gameInPlay=False
            fsmMgr.pushState(statePauseMenu.StatePauseMenu(None))
            return True

            
            
        
            
            
            

thisLevel=StateGame(None, None)
