import fsmMgr

import math

import vector
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import random

import screenInfo
import mouseInfo
import statePower

import stateDialog
import stateFlight
import statePutt
import tutorialProgress

import textDraw
import playerMgr
import options

import book
import miniSim

import widgets

AI_THINK_TIME=1.0

class StateAim(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.clicked=False

        self.stateTime=0

        self.prelimAim=False

        if options.gUnattended:
            global AI_THINK_TIME
            AI_THINK_TIME=0.01

    def init(self):
        tutorialProgress.gShownAim=True
        if self.parent:
            for obj in self.parent.physicsObjects:
                if obj.get("target",False):
                    cx,cy=obj["center"]

                    wx,wy=screenInfo.worldToScreenPos(cx,cy)
                    pygame.mouse.set_pos(wx,wy)

                    
                    break
                
        curPlayer=playerMgr.getCurrentPlayer()
        shotNum=curPlayer.scores.get(self.parent.level,0)+1
        shotMsg="%s on %s   SHOT %d"%(curPlayer.name,
                                      curPlayer.curPlanet["name"],
                                      shotNum)
        self.caption=textDraw.makeStringDL(30,10,
                                           4,
                                           shotMsg)
                            

        
            
        
    def update(self, dt):
        self.parent.update(dt)

        curPlayer=playerMgr.getCurrentPlayer()
        if not curPlayer.isHuman:
            self.stateTime+=dt/1000.0

            if not self.prelimAim:
                accuracy=self.computeAIMove()
                self.prelimAim=True
                self.angle=self.parent.angle

                self.angle+=random.uniform(-0.01, 0.01)

                self.aiWidth=2.0
                self.foundLandedAngle=False
                self.aiCandidateAngles=[]
                self.bestDist=-1
                self.thinkAngle=self.angle
                
            self.angle+=(self.thinkAngle-self.angle)*0.05

            if self.stateTime<AI_THINK_TIME and not self.foundLandedAngle:
                if len(self.aiCandidateAngles)==0:
                    numSteps=6
                    step=self.aiWidth/numSteps
                    deltas=range(-numSteps,numSteps+1)
                    random.shuffle(deltas)
                    for d in deltas:
                        self.aiCandidateAngles.append(self.thinkAngle+step*d)
                    self.aiWidth*=0.7
                
                testAIAngle=self.aiCandidateAngles.pop()
                predict=miniSim.evaluateLaunch(self.parent.physicsObjects,
                                               0.6,
                                               testAIAngle,
                                               100,
                                               0.0,
                                               0.1,
                                               playerMgr.getCurrentPlanet())
                if predict[0]:
                    self.foundLandedAngle=True
                    self.thinkAngle=testAIAngle
                else:
                    if (self.bestDist==-1 or
                        predict[4]<self.bestDist):
                        self.bestDist=predict[4]
                        self.thinkAngle=testAIAngle

            elif self.stateTime>AI_THINK_TIME:
                #accuracy=self.computeAIMove()
                #if accuracy is None:
                #    return
                self.parent.power=0.6
                self.parent.angle=self.angle
            
                glDeleteLists(self.caption, 1)
                fsmMgr.popState()
                fsmMgr.pushState(stateFlight.StateFlight(None))
                #statePutt.genericLaunch(self.parent, accuracy)
                statePutt.genericLaunch(self.parent, 1.0)

        else:
            # read the mouse - set the angle to the mouse
            mouseInfo.readMouse()
            mx=mouseInfo.gMouseX
            my=mouseInfo.gMouseY
            px,py=playerMgr.getCurrentPlanet()["center"]
            dx=mx-px
            dy=my-py
        
            self.angle=math.atan2(dy,dx)

    def computeAIMove(self):
        curPlayer=playerMgr.getCurrentPlayer()
        curPlanet=playerMgr.getCurrentPlanet()
        curLevel=self.parent.level
        curScore=curPlayer.scores[curLevel]

        

        gasGiantLocations=[]

        if playerMgr.noHumansLeft():
            bookData=book.levels.get(curLevel,{})
            planetData=bookData.get(curPlanet["name"],None)
            if planetData:
                shot=random.choice(planetData)
                self.parent.angle=shot[0]
                self.parent.power=shot[1]
                return 1.0
            
        
        
        shuffledPlanets=self.parent.physicsObjects[:]
        random.shuffle(shuffledPlanets)

        for obj in shuffledPlanets:
            if obj.get("gaseous",False):
                gasGiantLocations.append(obj["center"])

            
        for obj in shuffledPlanets:
            if obj.get("target",False):
                cx,cy=obj["center"]
                break
            if ((curScore>2) and
                (obj.get("orbits",False)) and
                (playerMgr.getCurrentPlanet()["name"]==
                 self.parent.physicsObjects[0]["name"])):
                #shoot for a moon if we can't go direct
                cx,cy=obj["center"]

        px,py=playerMgr.getCurrentPlanet()["center"]

        #straight shot to our chosen target
        dx=cx-px
        dy=cy-py

        ndx,ndy=vector.norm(dx,dy)

        
        anyBad=False
        if not options.gUnattended:
            for giant in gasGiantLocations:
                gx,gy=giant
                gdx,gdy=vector.norm(gx-px,gy-py)
                
                dot=vector.dot((ndx,ndy),(gdx,gdy))
    
                if (curPlanet.get("orbits", False) and
                    dot>0):
                    if random.randrange(1000)>5:
                        return None
                
                if dot>0.9:
                    anyBad=True

        self.parent.angle=math.atan2(dy,dx)

        if options.gUnattended:
            inside=0.2
            outside=1.5
        else:
            inside=0.25
            outside=0.7

        if anyBad:
            if random.randrange(2)>0:
                self.parent.angle+=random.uniform(inside,outside)
            else:
                self.parent.angle-=random.uniform(inside,outside)
                
        self.parent.power=random.uniform(.5,.7)

        #predict
        predict=miniSim.evaluateLaunch(shuffledPlanets,
                                       self.parent.power,
                                       self.parent.angle,
                                       100,
                                       0.0,
                                       0.1,
                                       playerMgr.getCurrentPlanet())




        if options.gUnattended:
            accuracyGuess=random.uniform(0.65, 1.0)
        else:
            accuracyGuess=random.uniform(0.85, 1.0)
        return accuracyGuess

        



    def handleMouseButton(self, bDown, buttonIndex):
        curPlayer=playerMgr.getCurrentPlayer()
        if not curPlayer.isHuman:
            return True
        
        
        #calibrate power next

        if bDown==True and buttonIndex==1:
            self.clicked=True
            return True
        if bDown==False and buttonIndex==1 and self.clicked:
            glDeleteLists(self.caption,1)
            self.parent.angle=self.angle
            fsmMgr.popState()
            fsmMgr.pushState(statePower.StatePower(self.parent))
            if (tutorialProgress.gTutorial and
                (not tutorialProgress.gShownPower)):
                fsmMgr.pushState(stateDialog.StateDialog(("Memo from Mr. Tigerbaum","""
The second step is to adjust the power
of the PACKAGE SLINGSHOT. This is done
by moving the mouse up and down.
Notice that this adjusts the power
meter on the right and the size of the
blue arrow. You may then click the left
button to proceed or the right button
to reaim.
""")))
            return True
        return False


    def draw(self):
        # first, let our parent draw the map
        self.parent.draw()
        
        # draw an arrow out of the planet in the direction of the
        # angle.
        # make sure to start it the radius of the current planet, and
        # go out from there.

        curPlan=playerMgr.getCurrentPlanet()
        
        curPlanRad=curPlan["radius"]
        curPlanX,curPlanY=curPlan["center"]
        
        arrowLength=70
        headLength=20
        headAngle=0.20

        glBegin(GL_LINES)

        
        glColor3f(0,1,0)
        arrowBaseX=curPlanX+math.cos(self.angle)*curPlanRad
        arrowBaseY=curPlanY+math.sin(self.angle)*curPlanRad

        arrowTipX=curPlanX+math.cos(self.angle)*(curPlanRad+arrowLength)
        arrowTipY=curPlanY+math.sin(self.angle)*(curPlanRad+arrowLength)

        glVertex2f(arrowBaseX,arrowBaseY)
        glVertex2f(arrowTipX,arrowTipY)

        arrowBarbX=curPlanX+math.cos(self.angle+headAngle)*(curPlanRad+arrowLength-headLength)
        arrowBarbY=curPlanY+math.sin(self.angle+headAngle)*(curPlanRad+arrowLength-headLength)
        glVertex2f(arrowTipX,arrowTipY)
        glVertex2f(arrowBarbX,arrowBarbY)

        arrowBarbX=curPlanX+math.cos(self.angle-headAngle)*(curPlanRad+arrowLength-headLength)
        arrowBarbY=curPlanY+math.sin(self.angle-headAngle)*(curPlanRad+arrowLength-headLength)
        glVertex2f(arrowTipX,arrowTipY)
        glVertex2f(arrowBarbX,arrowBarbY)
        glEnd()

        glCallList(self.caption)

        if fsmMgr.isTop(self):
            widgets.drawCursor()

