import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

from numpy import array
import numpy

import fsmMgr


import edgesBDG
import statePyWeekSplash

gEdgeList={}


#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR=(0.0, 0.55, 0.0)
FADE_TIME=0.4

SHOW_DELAY=2.0

class StateBDGSplash(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.clicked=False

        self.mRasterPos=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        gEdgeList=edgesBDG.lines

        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_COLOR_ARRAY)

        self.lineColors={}

        self.verts=[]

        self.lineIndices=[]
        
        lines=gEdgeList.keys()
        lines.sort()
        for y in lines:
            self.lineColors[y]=(0,0,0)
            for x in gEdgeList[y]:
                self.verts.append((float(x),
                                   float(y),
                                   0.0))
                self.lineIndices.append(y)
        
        self.vertArray=array(self.verts,'f')
        self.vertArrayString=self.vertArray.tostring()
        self.colorArray=array((DIM_COLOR)*len(self.verts), 'f')
        self.colorArrayString=self.colorArray.tostring()

        glVertexPointer(3, GL_FLOAT, 0, self.vertArrayString)
        glColorPointer(3, GL_FLOAT, 0, self.colorArrayString)

        self.numVerts=len(self.verts)

    def update(self, dt):

        dt=dt/1000.0

        self.splashTime+=dt

        self.mRasterPos-=self.rasterSpeed*dt

        if self.splashTime>SCAN_SPEED+SHOW_DELAY:
            self.done()

        if self.splashTime>SCAN_SPEED:
            return
        
        for y in self.lineColors.keys():
            distFromRaster=y-self.mRasterPos
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                self.lineColors[y]=DIM_COLOR
            elif secondsSinceRaster<0:
                self.lineColors[y]=(0.0, 0.0, 0.0)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb

                self.lineColors[y]=(interpR, interpG, interpB)
            

        for i,y in enumerate(self.lineIndices):
            cr,cg,cb=self.lineColors[y]
            self.colorArray[3*i]=cr
            self.colorArray[3*i+1]=cg
            self.colorArray[3*i+2]=cb
        self.colorArrayString=self.colorArray.tostring()
        glColorPointer(3, GL_FLOAT, 0, self.colorArrayString)
        

    def draw(self):
        glDrawArrays(GL_LINES, 0, self.numVerts)
        
    def handleMouseButton(self, bDown, buttonIndex):
        if bDown==True and buttonIndex==1:
            self.clicked=True
        if bDown==False and buttonIndex==1 and self.clicked:
            self.done()
        return True

    def handleKey(self, key, unicode):
        self.done()
        return True

    def done(self):
        fsmMgr.popState()
        fsmMgr.pushState(statePyWeekSplash.StatePyWeekSplash(None))

    

