import math
import sys
import random

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import mouseInfo

import statePlayAd
import stateDialog
import level1
import stateAim
import edgesMenu
import widgets
import tutorialProgress
import playerMgr
import stateMainMenu
import screenInfo
import fade       
import stateMainMenu
import stateNewGameMenuDeluxe

import tutorialProgress
import textDraw
import soundMgr

import levelIntros

MAXLEN=15

gHumanNames=["Philip J. Fry",
             "Leela",
             "Prof. Farnsworth",
             "Dr. Zoidberg",
             "David Bowman",
             "Luke Skywalker",
             "Michael Rennie"]

gAINames=["Bender",
          "C-3p0",
          "R2-d2",
          "IG-88",
          "HAL 9000",
          "SAL 9000",
          "Gort",
          "Robbie",
          "Vincent",
          "B.o.b."]

    
class StateEditPlayerDesc(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)

        self.clicked=False
        self.playerDesc=data

        self.left=40
        self.right=760
        self.top=560
        self.bottom=40
        self.pulse=0

        self.editingName=data.name

        self.widgets=[
                      widgets.Button(200,60,8,"RANDOM", self.pressRandomize),
                      widgets.Button(400,60,8,"CANCEL", self.done),
                      widgets.Button(600,60,8,"OK", self.pressOK),
                      ]

        myWidths=[175,175,175]

        assert(len(self.widgets)==len(myWidths))
        
        for i in range(len(self.widgets)):
            self.widgets[i].width=myWidths[i]

        for w in self.widgets:
            w.x=w.x-w.width/2
            w.makeDL()

        self.cursorPos=len(data.name)
        self.cursorInsert=True


    def handleKey(self, key, unicode):
        unicode=unicode.upper()
        
        if key==K_BACKSPACE:
            if self.cursorPos>0:
                firstBit=self.editingName[:self.cursorPos-1]
                secondBit=self.editingName[self.cursorPos:]
                self.editingName=firstBit+secondBit
                self.cursorPos -= 1
        elif key==K_DELETE:
            if len(self.editingName)>self.cursorPos:
                firstBit=self.editingName[:self.cursorPos]
                secondBit=self.editingName[self.cursorPos+1:]
                self.editingName=firstBit+secondBit
        elif key==K_LEFT:
            if self.cursorPos>0:
                self.cursorPos -= 1
        elif key==K_RIGHT:
            if self.cursorPos<len(self.editingName):
                self.cursorPos += 1
        elif key==K_HOME:
            self.cursorPos = 0
        elif key==K_END:
            self.cursorPos = len(self.editingName)
        elif key==K_INSERT:
            self.cursorInsert = not self.cursorInsert
        elif key==K_ESCAPE:
            self.done()
        elif key==K_RETURN:
            self.pressOK()
        elif textDraw.isPrintable(unicode) and len(self.editingName)<MAXLEN:
            nameList=list(self.editingName)

            if self.cursorInsert:
                nameList.insert(self.cursorPos, unicode)
            elif self.cursorPos==len(nameList):
                nameList.append(unicode)
            else:
                nameList[self.cursorPos]=unicode

            self.cursorPos+=1
            self.editingName=''.join(nameList)

        return True
        
        
        

    def update(self, dt):
        dt=dt/1000.0

        self.pulse+=dt

        mouseInfo.readMouse()
        for w in self.widgets:
            w.update(dt)
            
    def draw(self):
        glColor3f(0,0,0)
        glBegin(GL_QUADS)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()

        glColor3f(*fade.DIM_COLOR)
        glBegin(GL_LINE_LOOP)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()
        
        for w in self.widgets:
            w.draw()

        textDraw.drawCenteredString(400, 525,
                                    fade.DIM_COLOR,
                                    8,
                                    "EDIT PLAYER NAME")

        textDraw.drawCenteredStringWithCursor(400, 225,
                                              fade.DIM_COLOR,
                                              8,
                                              self.editingName,
                                              self.cursorPos,
                                              self.cursorInsert,
                                              self.pulse)

        if fsmMgr.isTop(self):
            widgets.drawCursor()


            

    def handleMouseButton(self, bDown, buttonIndex):
        mouseInfo.readMouse()
        if bDown and buttonIndex==1:
            for w in self.widgets:
                if w.isMouseInMe():
                    soundMgr.playSound("ping")
                    w.callbackFunc()
        return True

    def done(self):
        fsmMgr.popState()

    def pressOK(self):
        self.playerDesc.name=self.editingName
        self.done()

    def pressRandomize(self):
        count=0
        
        while 1:
            if self.playerDesc.isHuman:
                testName=random.choice(gHumanNames)
            else:
                testName=random.choice(gAINames)

            if count>100:
                testName+=" %d"%(count-100)

            if not stateNewGameMenuDeluxe.isNameInUse(testName):
                break

            count += 1
                
        self.editingName=testName

        self.cursorPos=len(self.editingName)

    

