import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import textDraw
import stateTitleSplash

import edgesPW4_1 
import edgesPW4_2 

from numpy import array


#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR_1=(0.0, 0.75, 0.0, 1.0)
DIM_COLOR_2=(0.0, 0.55, 0.0, 1.0)

FADE_TIME=0.4

SHOW_DELAY=3.5



class StatePyWeekSplash(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.clicked=False

        self.mRasterPosns=[600,600]
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        self.dimColors=[DIM_COLOR_2,
                        DIM_COLOR_1]

        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_COLOR_ARRAY)

        self.lineColors=[{}, {}]

        self.verts=[]

        # pass,line number
        self.lineIndices=[]

        gEdgeLists=[edgesPW4_1.lines,
                    edgesPW4_2.lines]
        
        for i in range(2):
            lines=gEdgeLists[i].keys()
            lines.sort()
            for y in lines:
                self.lineColors[i][y]=(0,0,0)
                for x in gEdgeLists[i][y]:
                    self.verts.append((float(x),
                                       float(y),
                                       0.0))
                    self.lineIndices.append((i,y))

        self.vertArray=array(self.verts,'f')
        self.vertArrayString=self.vertArray.tostring()
        self.colorArray=array((0.0, 0.0, 0.0, 0.0)*len(self.verts), 'f')
        self.colorArrayString=self.colorArray.tostring()

        glVertexPointer(3, GL_FLOAT, 0, self.vertArrayString)
        glColorPointer(4, GL_FLOAT, 0, self.colorArrayString)

        self.numVerts=len(self.verts)

        self.delays=[0,SCAN_SPEED*0.5]

        self.textDLs=[textDraw.makeCenteredStringDL(400,500,10,"An entry in"),
                      textDraw.makeCenteredStringDL(400,175, 6,"PyWeek 4 game development challenge"),
                      textDraw.makeCenteredStringDL(400,125, 6,"http://www.pyweek.org/4/"),
                      textDraw.makeCenteredStringDL(400, 75, 6,"entry BDG_07_04")]





    def update(self, dt):
        dt=dt/1000.0
        self.splashTime+=dt

        for i in range(2):
            if self.splashTime>self.delays[i]:
                self.mRasterPosns[i]-=self.rasterSpeed*dt

        if self.splashTime > SCAN_SPEED+SHOW_DELAY:
            self.done()

        if self.splashTime > SCAN_SPEED+max(self.delays):
            return

        for i in range(2):
            for y in self.lineColors[i].keys():
                distFromRaster = y-self.mRasterPosns[i]
                secondsSinceRaster=distFromRaster/self.rasterSpeed

                if secondsSinceRaster>FADE_TIME:
                    self.lineColors[i][y]=self.dimColors[i]
                elif secondsSinceRaster<0:
                    self.lineColors[i][y]=(0.0, 0.0, 0.0, 0.0)
                else:
                    br,bg,bb=BRIGHT_COLOR
                    dr,dg,db,da=self.dimColors[i]
                    
                    fracSinceRaster=secondsSinceRaster/FADE_TIME

                    interpR=(dr-br)*fracSinceRaster+br
                    interpG=(dg-bg)*fracSinceRaster+bg
                    interpB=(db-bb)*fracSinceRaster+bb
                    
                    self.lineColors[i][y]=(interpR, interpG, interpB, 1.0)

        for vertIndex, (pageIndex, lineNum) in enumerate(self.lineIndices):
            cr,cg,cb,ca=self.lineColors[pageIndex][lineNum]
            self.colorArray[4*vertIndex]=cr
            self.colorArray[4*vertIndex+1]=cg
            self.colorArray[4*vertIndex+2]=cb
            self.colorArray[4*vertIndex+3]=ca
        self.colorArrayString=self.colorArray.tostring()
        glColorPointer(4, GL_FLOAT, 0, self.colorArrayString)
            
                    
            

    def draw(self):
        glDrawArrays(GL_LINES, 0, self.numVerts)

        c=self.computeColor(0.0, self.splashTime)

        glColor3f(*c)

        for dl in self.textDLs:
            glCallList(dl)
        
        
    def handleMouseButton(self, bDown, buttonIndex):
        #calibrate power next

        if bDown==True and buttonIndex==1:
            self.clicked=True
        if bDown==False and buttonIndex==1 and self.clicked:
            self.done()
        return True

    def handleKey(self, key, unicode):
        self.done()
        return True

    def done(self):
        for dl in self.textDLs:
            glDeleteLists(dl,1)
            
        fsmMgr.popState()
        fsmMgr.pushState(stateTitleSplash.StateTitleSplash(None))


    def computeColor(self, initTime, timeNow):
        elapsedTime=timeNow-initTime
        
        if elapsedTime>FADE_TIME:
            return DIM_COLOR_2[:3]
        else:
            br,bg,bb=BRIGHT_COLOR
            dr,dg,db,da=DIM_COLOR_2

            fracSinceRaster=elapsedTime/FADE_TIME

            interpR=(dr-br)*fracSinceRaster+br
            interpG=(dg-bg)*fracSinceRaster+bg
            interpB=(db-bb)*fracSinceRaster+bb

            return (interpR, interpG, interpB)
                
                
        
